/* -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
/***************************************************************************
 *            ThreadedItemSource.cpp
 *
 *  FUPPES - Free UPnP Entertainment Service
 *
 *  Copyright (C) 2011 Ulrich Völkel <u-voelkel@users.sourceforge.net>
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "ItemSource.h"
#include "../Common/Exception.h"
#include "../Log.h"
#include "../SharedConfig.h"

using namespace fuppes;


ItemSource::ItemSource(fuppesLibHandle handle, plugin_info* info)
:	CPlugin(handle, info) 
{
  m_open = NULL;
  m_next = NULL;
  m_close = NULL;
}

ItemSource::ItemSource(ItemSource* plugin)
:CPlugin(plugin->m_handle, &plugin->m_pluginInfo) 
{
  m_open = plugin->m_open;
  m_next = plugin->m_next;
  m_close = plugin->m_close;
}

bool ItemSource::initPlugin()
{
  m_open = NULL;
  m_next = NULL;
  m_close = NULL;
  
	m_open = (itemSourceOpen_t)FuppesGetProcAddress(m_handle, "fuppes_source_open");
	if(NULL == m_open) {
		return false;
	}

  m_next = (itemSourceNext_t)FuppesGetProcAddress(m_handle, "fuppes_source_next");
	if(NULL == m_next) {
		return false;
	}

  m_close = (itemSourceClose_t)FuppesGetProcAddress(m_handle, "fuppes_source_close");
	if(NULL == m_close) {
		return false;
	}  
  
	return true;
}

bool ItemSource::open(std::string name)
{
  if(NULL == m_open)
    return false;

  return ((m_open(&m_pluginInfo, name.c_str())) == 0);
}

bool ItemSource::next(struct metadata_t* metadata)
{
  if(NULL == m_next)
    return false;

  return ((m_next(&m_pluginInfo, metadata)) == 0);
}

void ItemSource::close()
{
  if(NULL == m_close)
    return;

  m_close(&m_pluginInfo);
}
